{ ------------------------------------------------------------------------------------------------------------------- }
{                                                                                                                     }
{ Die Unit stellt einige Grundlegende AVLBume zur Verfgung:                                                         }
{                                                                                                                     }
{  type TIntAVLbaum = class(TAVLbaum)                                                                                 }
{       [ ist ein AVLbaum auf Integer-Basis ]                                                                         }
{                                                                                                                     }
{  type TStringAVLbaum = class(TAVLbaum)                                                                              }
{       [ ist ein AVLbaum auf String-Basis ]                                                                          }
{                                                                                                                     }
{  type TPointAVLbaum = class(TAVLbaum)                                                                               }
{       [ ist ein AVLBaum auf Point-Basis ]                                                                          }
{                                                                                                                     }
{  type TAdressAVLbaum = class(TAVLbaum)                                                                              }
{       [ ist ein AVLbaum auf Adress-Basis ]                                                                          }
{                                                                                                                     }
{                                                                                                                     }
{  Weitere Baumtypen sind denkbar. Damit die Bume AVLbume sind, mssen sie ber die Methoden                        }
{                                                                                                                     }
{      - equal                                                                                                        }
{      - higher                                                                                                       }
{                                                                                                                     }
{  verfgen. Zur Anzeige in Formularfeldern wird die Methode    WurzelinhaltToString     bentogt.                    }
{  Andere Methoden sind ebenfalls denkbar.                                                                            }
{                                                                                                                     }
{ --------------------------------------------------------------- erweitertes Tool zum AVLbaum, (C) 2002 D. Garmann - }

unit avlbaume;

interface

uses suchbaum, avlbaum;

type TIntAVLBaum = class(TAVLBaum)
       private
       Wurzelinhalt: integer;
       public
       constructor Create(w: Integer);
       function equal(e: TSuchBaum): boolean; override;            { diese beiden Routinen stammen noch von TSuchbaum }
       function higher(e: TSuchBaum): boolean; override;
       function WurzelinhaltToString: string; override;
     end;

     TMyString       = String[20];
     TStringAVLBaum = class(TAVLBaum)
       private
       Wurzelinhalt: TMyString;
       public
       constructor Create(w: TMyString);
       function equal(e: TSuchBaum): boolean; override;            { diese beiden Routinen stammen noch von TSuchbaum }
       function higher(e: TSuchBaum): boolean; override;
       function WurzelinhaltToString: string; override;
     end;

     TMyPoint        = record x,y: integer; end;
     TPointAVLBaum = class(TAVLBaum)
       private
       Wurzelinhalt: TMyPoint;
       public
       constructor Create(x,y: integer);
       function equal(e: TSuchBaum): boolean; override;            { diese beiden Routinen stammen noch von TSuchbaum }
       function higher(e: TSuchBaum): boolean; override;
       function WurzelinhaltToString: string; override;
     end;

     TMyAdress      = record name, vorname, plz, ort: TMyString; end;
     TAdressAVLBaum = class(TAVLBaum)
       private
       Wurzelinhalt: TMyAdress;
       public
       constructor Create(n,v,p,o: TMyString);
       function equal(e: TSuchBaum): boolean; override;            { diese beiden Routinen stammen noch von TSuchbaum }
       function higher(e: TSuchBaum): boolean; override;
       function WurzelinhaltToString: string; override;
     end;


implementation

{ ------------------------  Routinen fr TIntAVLBaum  -------------------------------------------------------------- }

constructor TIntAVLBaum.Create(w: Integer);
begin
  inherited create;
  Wurzelinhalt:= w;
end;

function TIntAVLBaum.equal(e: TSuchBaum): boolean;
begin
  equal:= (Wurzelinhalt = (e as TIntAVLBaum).Wurzelinhalt);
end;

function TIntAVLBaum.higher(e: TSuchBaum): boolean;
begin
  higher:= (Wurzelinhalt > (e as TIntAVLBaum).Wurzelinhalt);
end;

function TIntAVLBaum.WurzelinhaltToString: string;
var s: string;
    h, b: string;
begin
  str(Wurzelinhalt, s);
  str(Balance, b);
  str(BaumHoehe, h);
  WurzelinhaltToString:= s+' H:'+h+' B:'+b;
end;


{ ------------------------  Routinen fr TStringAVLBaum  ----------------------------------------------------------- }

constructor TStringAVLBaum.Create(w: TMyString);
begin
  inherited create;
  Wurzelinhalt:= w;
end;

function TStringAVLBaum.equal(e: TSuchBaum): boolean;
begin
  equal:= (Wurzelinhalt = (e as TStringAVLBaum).Wurzelinhalt);
end;

function TStringAVLBaum.higher(e: TSuchBaum): boolean;
begin
  higher:= (Wurzelinhalt > (e as TStringAVLBaum).Wurzelinhalt);
end;

function TStringAVLBaum.WurzelinhaltToString: string;
var h, b: string;
begin
  str(Balance, b);
  str(BaumHoehe, h);
  WurzelinhaltToString:= Wurzelinhalt+' H:'+h+' B:'+b;
end;


{ ------------------------  Routinen fr TPointAVLBaum  ------------------------------------------------------------ }

constructor TPointAVLBaum.Create(x,y: Integer);
begin
  inherited create;
  Wurzelinhalt.x:= x;
  Wurzelinhalt.y:= y;
end;

function TPointAVLBaum.equal(e: TSuchBaum): boolean;
begin
  equal:= (Wurzelinhalt.x = (e as TPointAVLBaum).Wurzelinhalt.x) and
          (Wurzelinhalt.y = (e as TPointAVLBaum).Wurzelinhalt.y);
end;

function TPointAVLBaum.higher(e: TSuchBaum): boolean; { Sortierung nach x, dann nach y-Koordinate                    }
begin
  higher:= (Wurzelinhalt.x > (e as TPointAVLBaum).Wurzelinhalt.x) and
           (Wurzelinhalt.y > (e as TPointAVLBaum).Wurzelinhalt.y);
end;

function TPointAVLBaum.WurzelinhaltToString: string;
var sx,sy: string;
    h, b: string;
begin
  str(Wurzelinhalt.x, sx);
  str(Wurzelinhalt.y, sy);
  str(Balance, b);
  str(BaumHoehe, h);
  WurzelinhaltToString:= '('+sx+'|'+sy+')'+' H:'+h+' B:'+b;
end;


{ ------------------------  Routinen fr TAdressAVLBaum  ----------------------------------------------------------- }

constructor TAdressAVLBaum.Create(n,v,p,o: TMyString);
begin
  inherited create;
  Wurzelinhalt.name:= n;
  Wurzelinhalt.vorname:= v;
  Wurzelinhalt.plz:= p;
  Wurzelinhalt.ort:= o;
end;

function TAdressAVLBaum.equal(e: TSuchBaum): boolean;
begin
  equal:= (Wurzelinhalt.name = (e as TAdressAVLBaum).Wurzelinhalt.name) and
          (Wurzelinhalt.vorname = (e as TAdressAVLBaum).Wurzelinhalt.vorname) and
          (Wurzelinhalt.plz = (e as TAdressAVLBaum).Wurzelinhalt.plz) and
          (Wurzelinhalt.ort = (e as TAdressAVLBaum).Wurzelinhalt.ort);
end;

function TAdressAVLBaum.higher(e: TSuchBaum): boolean; { Sortierung nach Name, dann nach Vorname, dann nach PLZ.     }
begin
  if (Wurzelinhalt.name > (e as TAdressAVLBaum).Wurzelinhalt.name)
  then higher:= true
  else if (Wurzelinhalt.name = (e as TAdressAVLBaum).Wurzelinhalt.name)
       then begin
              if (Wurzelinhalt.vorname > (e as TAdressAVLBaum).Wurzelinhalt.vorname)
              then higher:= true
              else if (Wurzelinhalt.vorname = (e as TAdressAVLBaum).Wurzelinhalt.vorname)
                   then begin
                          if (Wurzelinhalt.plz > (e as TAdressAVLBaum).Wurzelinhalt.plz)
                          then higher:= true
                          else higher:= false;
                        end
                   else higher:= false;
            end
       else higher:= false;
end;

function TAdressAVLBaum.WurzelinhaltToString: string;
var h, b: string;
begin
  str(Balance, b);
  str(BaumHoehe, h);
  WurzelinhaltToString:= Wurzelinhalt.vorname+' '+Wurzelinhalt.name+', wohnhaft in '
                         +Wurzelinhalt.plz+' '+Wurzelinhalt.ort+' H:'+h+' B:'+b;
end;



end.
