{ ------------------------------------------------------------------------------------------------------------------- }
{                                                                                                                     }
{ Die Unit stellt einige Grundlegende Suchbume zur Verfgung:                                                        }
{                                                                                                                     }
{  type TIntSuchbaum = class(TSuchbaum)                                                                               }
{       [ ist ein Suchbaum auf Integer-Basis ]                                                                        }
{                                                                                                                     }
{  type TStringSuchbaum = class(TSuchbaum)                                                                            }
{       [ ist ein Suchbaum auf Integer-Basis ]                                                                        }
{                                                                                                                     }
{  type TPointSuchbaum = class(TSuchbaum)                                                                             }
{       [ ist ein Suchbaum auf Integer-Basis ]                                                                        }
{                                                                                                                     }
{  type TAdressSuchbaum = class(TSuchbaum)                                                                            }
{       [ ist ein Suchbaum auf Integer-Basis ]                                                                        }
{                                                                                                                     }
{                                                                                                                     }
{  Weitere Baumtypen sind denkbar. Damit die Bume Suchbume sind, mssen sie ber die Methoden                       }
{                                                                                                                     }
{      - equal                                                                                                        }
{      - higher                                                                                                       }
{                                                                                                                     }
{  verfgen. Zur Anzeige in Formularfeldern wird die Methode    WurzelinhaltToString     bentogt.                    }
{  Andere Methoden sind ebenfalls denkbar.                                                                            }
{                                                                                                                     }
{ -------------------------------------------------------------- erweitertes Tool zum Suchbaum, (C) 2001 D. Garmann - }

unit Sbaume;

interface

uses suchbaum;

type TIntSuchbaum = class(TSuchbaum)
       private
       Wurzelinhalt: integer;
       public
       constructor Create(w: Integer);
       function equal(e: TSuchbaum): boolean; override;
       function higher(e: TSuchbaum): boolean; override;
       function WurzelinhaltToString: string; override;
     end;

     TMyString       = String[20];
     TStringSuchbaum = class(TSuchbaum)
       private
       Wurzelinhalt: TMyString;
       public
       constructor Create(w: TMyString);
       function equal(e: TSuchbaum): boolean; override;
       function higher(e: TSuchbaum): boolean; override;
       function WurzelinhaltToString: string; override;
     end;

     TMyPoint        = record x,y: integer; end;
     TPointSuchbaum = class(TSuchbaum)
       private
       Wurzelinhalt: TMyPoint;
       public
       constructor Create(x,y: integer);
       function equal(e: TSuchbaum): boolean; override;
       function higher(e: TSuchbaum): boolean; override;
       function WurzelinhaltToString: string; override;
     end;

     TMyAdress      = record name, vorname, plz, ort: TMyString; end;
     TAdressSuchbaum = class(TSuchbaum)
       private
       Wurzelinhalt: TMyAdress;
       public
       constructor Create(n,v,p,o: TMyString);
       function equal(e: TSuchbaum): boolean; override;
       function higher(e: TSuchbaum): boolean; override;
       function WurzelinhaltToString: string; override;
     end;


implementation

{ ------------------------  Routinen fr TIntSuchbaum  -------------------------------------------------------------- }

constructor TIntSuchbaum.Create(w: Integer);
begin
  inherited create;
  Wurzelinhalt:= w;
end;

function TIntSuchbaum.equal(e: TSuchbaum): boolean;
begin
  equal:= (Wurzelinhalt = (e as TIntSuchbaum).Wurzelinhalt);
end;

function TIntSuchbaum.higher(e: TSuchbaum): boolean;
begin
  higher:= (Wurzelinhalt > (e as TIntSuchbaum).Wurzelinhalt);
end;

function TIntSuchbaum.WurzelinhaltToString: string;
var s: string;
begin
  str(Wurzelinhalt, s);
  WurzelinhaltToString:= s;
end;


{ ------------------------  Routinen fr TStringSuchbaum  ----------------------------------------------------------- }

constructor TStringSuchbaum.Create(w: TMyString);
begin
  inherited create;
  Wurzelinhalt:= w;
end;

function TStringSuchbaum.equal(e: TSuchbaum): boolean;
begin
  equal:= (Wurzelinhalt = (e as TStringSuchbaum).Wurzelinhalt);
end;

function TStringSuchbaum.higher(e: TSuchbaum): boolean;
begin
  higher:= (Wurzelinhalt > (e as TStringSuchbaum).Wurzelinhalt);
end;

function TStringSuchbaum.WurzelinhaltToString: string;
begin
  WurzelinhaltToString:= Wurzelinhalt;
end;


{ ------------------------  Routinen fr TPointSuchbaum  ------------------------------------------------------------ }

constructor TPointSuchbaum.Create(x,y: Integer);
begin
  inherited create;
  Wurzelinhalt.x:= x;
  Wurzelinhalt.y:= y;
end;

function TPointSuchbaum.equal(e: TSuchbaum): boolean;
begin
  equal:= (Wurzelinhalt.x = (e as TPointSuchbaum).Wurzelinhalt.x) and
          (Wurzelinhalt.y = (e as TPointSuchbaum).Wurzelinhalt.y);
end;

function TPointSuchbaum.higher(e: TSuchbaum): boolean; { Sortierung nach x, dann nach y-Koordinate                    }
begin
  higher:= (Wurzelinhalt.x > (e as TPointSuchbaum).Wurzelinhalt.x) and
           (Wurzelinhalt.y > (e as TPointSuchbaum).Wurzelinhalt.y);
end;

function TPointSuchbaum.WurzelinhaltToString: string;
var sx,sy: string;
begin
  str(Wurzelinhalt.x, sx);
  str(Wurzelinhalt.y, sy);
  WurzelinhaltToString:= '('+sx+'|'+sy+')';
end;


{ ------------------------  Routinen fr TAdressSuchbaum  ----------------------------------------------------------- }

constructor TAdressSuchbaum.Create(n,v,p,o: TMyString);
begin
  inherited create;
  Wurzelinhalt.name:= n;
  Wurzelinhalt.vorname:= v;
  Wurzelinhalt.plz:= p;
  Wurzelinhalt.ort:= o;
end;

function TAdressSuchbaum.equal(e: TSuchbaum): boolean;
begin
  equal:= (Wurzelinhalt.name = (e as TAdressSuchbaum).Wurzelinhalt.name) and
          (Wurzelinhalt.vorname = (e as TAdressSuchbaum).Wurzelinhalt.vorname) and
          (Wurzelinhalt.plz = (e as TAdressSuchbaum).Wurzelinhalt.plz) and
          (Wurzelinhalt.ort = (e as TAdressSuchbaum).Wurzelinhalt.ort);
end;

function TAdressSuchbaum.higher(e: TSuchbaum): boolean; { Sortierung nach Name, dann nach Vorname, dann nach PLZ.     }
begin
  if (Wurzelinhalt.name > (e as TAdressSuchbaum).Wurzelinhalt.name)
  then higher:= true
  else if (Wurzelinhalt.name = (e as TAdressSuchbaum).Wurzelinhalt.name)
       then begin
              if (Wurzelinhalt.vorname > (e as TAdressSuchbaum).Wurzelinhalt.vorname)
              then higher:= true
              else if (Wurzelinhalt.vorname = (e as TAdressSuchbaum).Wurzelinhalt.vorname)
                   then begin
                          if (Wurzelinhalt.plz > (e as TAdressSuchbaum).Wurzelinhalt.plz)
                          then higher:= true
                          else higher:= false;
                        end
                   else higher:= false;
            end
       else higher:= false;
end;

function TAdressSuchbaum.WurzelinhaltToString: string;
begin
  WurzelinhaltToString:= Wurzelinhalt.vorname+' '+Wurzelinhalt.name+', wohnhaft in '
                         +Wurzelinhalt.plz+' '+Wurzelinhalt.ort;
end;



end.
